@@ -27,13 +27,10 @@
 #include "mail-storage-private.h"
 
 #include "antispam-plugin.h"
-#include "signature.h"
 
-static const char *reaver_binary = "/bin/false";
-static char **extra_args = NULL;
-static int extra_args_num = 0;
 
-static int call_reaver(const char *signature, enum classification wanted)
+static int call_reaver(const struct antispam_config *cfg,
+		       const char *signature, enum classification wanted)
 {
 	pid_t pid;
 	const char *class_arg;
@@ -67,7 +64,8 @@ static int call_reaver(const char *signature, enum classification wanted)
 		/*
 		 * Reaver wants the mail but only needs the cache ID
 		 */
-		write(pipes[1], signature_hdr, strlen(signature_hdr));
+		write(pipes[1], cfg->crm.sigcfg.signature_hdr,
+			strlen(cfg->crm.sigcfg.signature_hdr));
 		write(pipes[1], ": ", 2);
 		write(pipes[1], signature, strlen(signature));
 		write(pipes[1], "\r\n\r\n", 4);
@@ -85,7 +83,7 @@ static int call_reaver(const char *signature, enum classification wanted)
 		int fd = open("/dev/null", O_RDONLY);
 		char **argv;
 		/* 2 fixed, extra, terminating NULL */
-		int sz = sizeof(char *) * (2 + extra_args_num + 1);
+		int sz = sizeof(char *) * (2 + cfg->crm.extra_args_num + 1);
 		int i;
 
 		argv = i_malloc(sz);
@@ -107,18 +105,31 @@ static int call_reaver(const char *signature, enum classification wanted)
 			exit(1);
 		close(fd);
 
-		argv[0] = (char *)reaver_binary;
+		argv[0] = (char *)cfg->crm.reaver_binary;
 		argv[1] = (char *)class_arg;
 
-		for (i = 0; i < extra_args_num; i++)
-			argv[i + 2] = (char *)extra_args[i];
+		for (i = 0; i < cfg->crm.extra_args_num; i++)
+			argv[i + 2] = (char *)cfg->crm.extra_args[i];
 
-		debugv(argv);
+		debugv(&cfg->dbgcfg, argv);
 
-		execv(reaver_binary, argv);
+		t_push();
+		for (i = 0; i < cfg->crm.extra_env_num; i++) {
+			char *name, *value;
+			name = t_strdup_noconst(cfg->crm.extra_env[i]);
+			value = strchr(name, '=');
+			if (value) {
+				*value = '\0';
+				value++;
+			}
+			setenv(name, value, 1);
+		}
+		t_pop();
+
+		execv(cfg->crm.reaver_binary, argv);
 		/* fall through if reaver can't be found */
-                debug("executing %s failed: %d (uid=%d, gid=%d)",
-			reaver_binary, errno, getuid(), getgid());
+		debug(&cfg->dbgcfg, "executing %s failed: %d (uid=%d, gid=%d)",
+			cfg->crm.reaver_binary, errno, getuid(), getgid());
 		exit(127);
 		/* not reached */
 		return -1;
@@ -129,8 +140,9 @@ struct antispam_transaction_context {
 	struct siglist *siglist;
 };
 
-struct antispam_transaction_context *
-backend_start(struct mailbox *box __attr_unused__)
+static struct antispam_transaction_context *
+backend_start(const struct antispam_config *cfg ATTR_UNUSED,
+	      struct mailbox *box ATTR_UNUSED)
 {
 	struct antispam_transaction_context *ast;
 
@@ -139,20 +151,22 @@ backend_start(struct mailbox *box __attr_unused__)
 	return ast;
 }
 
-void backend_rollback(struct antispam_transaction_context *ast)
+static void backend_rollback(const struct antispam_config *cfg ATTR_UNUSED,
+			     struct antispam_transaction_context *ast)
 {
 	signature_list_free(&ast->siglist);
 	i_free(ast);
 }
 
-int backend_commit(struct mailbox_transaction_context *ctx,
-		   struct antispam_transaction_context *ast)
+static int backend_commit(const struct antispam_config *cfg,
+			  struct mailbox_transaction_context *ctx,
+			  struct antispam_transaction_context *ast)
 {
 	struct siglist *item = ast->siglist;
 	int ret = 0;
 
 	while (item) {
-		if (call_reaver(item->sig, item->wanted)) {
+		if (call_reaver(cfg, item->sig, item->wanted)) {
 			ret = -1;
 			mail_storage_set_error(ctx->box->storage,
 					       ME(NOTPOSSIBLE)
@@ -167,37 +181,55 @@ int backend_commit(struct mailbox_transaction_context *ctx,
 	return ret;
 }
 
-int backend_handle_mail(struct mailbox_transaction_context *t,
-			struct antispam_transaction_context *ast,
-			struct mail *mail, enum classification want)
+static int backend_handle_mail(const struct antispam_config *cfg,
+			       struct mailbox_transaction_context *t,
+			       struct antispam_transaction_context *ast,
+			       struct mail *mail, enum classification want)
 {
-	return signature_extract_to_list(t, mail, &ast->siglist, want);
+	return signature_extract_to_list(&cfg->crm.sigcfg, t, mail, &ast->siglist, want);
 }
 
-void backend_init(pool_t pool)
+static void backend_init(struct antispam_config *cfg,
+			 const char *(getenv)(const char *env, void *data),
+			 void *getenv_data)
 {
 	const char *tmp;
 	int i;
 
-	tmp = get_setting("CRM_BINARY");
+	tmp = getenv("CRM_BINARY", getenv_data);
 	if (tmp) {
-		reaver_binary = tmp;
-		debug("reaver binary set to %s\n", tmp);
+		cfg->crm.reaver_binary = tmp;
+		debug(&cfg->dbgcfg, "reaver binary set to %s\n", tmp);
+	} else
+		cfg->crm.reaver_binary = "/bin/false";
+
+	tmp = getenv("CRM_ARGS", getenv_data);
+	if (tmp) {
+		cfg->crm.extra_args = p_strsplit(cfg->mem_pool, tmp, ";");
+		cfg->crm.extra_args_num = str_array_length(
+					(const char *const *)cfg->crm.extra_args);
+		for (i = 0; i < cfg->crm.extra_args_num; i++)
+			debug(&cfg->dbgcfg, "reaver extra arg %s\n",
+			      cfg->crm.extra_args[i]);
 	}
 
-	tmp = get_setting("CRM_ARGS");
+	tmp = getenv("CRM_ENV", getenv_data);
 	if (tmp) {
-		extra_args = p_strsplit(pool, tmp, ";");
-		extra_args_num = str_array_length(
-					(const char *const *)extra_args);
-		for (i = 0; i < extra_args_num; i++)
-			debug("reaver extra arg %s\n",
-			      extra_args[i]);
+		cfg->crm.extra_env = p_strsplit(cfg->mem_pool, tmp, ";");
+		cfg->crm.extra_env_num = str_array_length(
+					(const char *const *)cfg->crm.extra_env);
+		for (i = 0; i < cfg->crm.extra_env_num; i++)
+			debug(&cfg->dbgcfg, "reaver env %s\n",
+			      cfg->crm.extra_env[i]);
 	}
 
-	signature_init();
+	signature_init(&cfg->crm.sigcfg, &cfg->dbgcfg, getenv, getenv_data);
 }
 
-void backend_exit(void)
-{
-}
+struct backend crm114_backend = {
+	.init = backend_init,
+	.handle_mail = backend_handle_mail,
+	.start = backend_start,
+	.rollback = backend_rollback,
+	.commit = backend_commit,
+};
