@@ -27,16 +27,10 @@
 #include "mail-storage-private.h"
 
 #include "antispam-plugin.h"
-#include "signature.h"
 
-static const char *dspam_binary = "/usr/bin/dspam";
-static const char *dspam_result_header = NULL;
-static char **dspam_result_bl = NULL;
-static int dspam_result_bl_num = 0;
-static char **extra_args = NULL;
-static int extra_args_num = 0;
 
-static int call_dspam(const char *signature, enum classification wanted)
+static int call_dspam(const struct antispam_config *cfg,
+		      const char *signature, enum classification wanted)
 {
 	pid_t pid;
 	const char *class_arg;
@@ -58,7 +52,8 @@ static int call_dspam(const char *signature, enum classification wanted)
 	 * non-zero exit code on errors so we treat it as an error
 	 * if it logged anything to stderr.
 	 */
-	pipe(pipes);
+	if (pipe(pipes) < 0)
+		return -1;
 
 	pid = fork();
 	if (pid < 0)
@@ -90,7 +85,7 @@ static int call_dspam(const char *signature, enum classification wanted)
 
 			if (readsize > 0) {
 				buf[readsize] = '\0';
-				debug("dspam error: %s\n", buf);
+				debug(&cfg->dbgcfg, "dspam error: %s\n", buf);
 			}
 		} while (readsize == -2 || readsize > 0);
 
@@ -110,7 +105,7 @@ static int call_dspam(const char *signature, enum classification wanted)
 		int fd = open("/dev/null", O_RDONLY);
 		char **argv;
 		/* 4 fixed args, extra args, terminating NULL */
-		int sz = sizeof(char *) * (4 + extra_args_num + 1);
+		int sz = sizeof(char *) * (4 + cfg->dspam.extra_args_num + 1);
 		int i;
 
 		argv = i_malloc(sz);
@@ -132,25 +127,36 @@ static int call_dspam(const char *signature, enum classification wanted)
 			exit(1);
 		close(fd);
 
-		argv[0] = (char *)dspam_binary;
+		argv[0] = (char *)cfg->dspam.binary;
 		argv[1] = "--source=error";
 		argv[2] = (char *)class_arg;
 		argv[3] = (char *)sign_arg;
 
-		for (i = 0; i < extra_args_num; i++)
-			argv[i + 4] = (char *)extra_args[i];
+		for (i = 0; i < cfg->dspam.extra_args_num; i++)
+			argv[i + 4] = (char *)cfg->dspam.extra_args[i];
 
-#ifdef DEBUG_SYSLOG
 		/*
 		 * not good with stderr debuggin since we then write to
 		 * stderr which our parent takes as a bug
 		 */
-		debugv(argv);
-#endif
+		debugv_not_stderr(&cfg->dbgcfg, argv);
 
-		execv(dspam_binary, argv);
-		debug("executing %s failed: %d (uid=%d, gid=%d)",
-			dspam_binary, errno, getuid(), getgid());
+		t_push();
+		for (i = 0; i < cfg->dspam.extra_env_num; i++) {
+			char *name, *value;
+			name = t_strdup_noconst(cfg->dspam.extra_env[i]);
+			value = strchr(name, '=');
+			if (value) {
+				*value = '\0';
+				value++;
+			}
+			setenv(name, value, 1);
+		}
+		t_pop();
+
+		execv(cfg->dspam.binary, argv);
+		debug(&cfg->dbgcfg, "executing %s failed: %d (uid=%d, gid=%d)",
+			cfg->dspam.binary, errno, getuid(), getgid());
 		/* fall through if dspam can't be found */
 		exit(127);
 		/* not reached */
@@ -162,8 +168,9 @@ struct antispam_transaction_context {
 	struct siglist *siglist;
 };
 
-struct antispam_transaction_context *
-backend_start(struct mailbox *box __attr_unused__)
+static struct antispam_transaction_context *
+backend_start(const struct antispam_config *cfg ATTR_UNUSED,
+	      struct mailbox *box ATTR_UNUSED)
 {
 	struct antispam_transaction_context *ast;
 
@@ -172,20 +179,22 @@ backend_start(struct mailbox *box __attr_unused__)
 	return ast;
 }
 
-void backend_rollback(struct antispam_transaction_context *ast)
+static void backend_rollback(const struct antispam_config *cfg ATTR_UNUSED,
+			     struct antispam_transaction_context *ast)
 {
 	signature_list_free(&ast->siglist);
 	i_free(ast);
 }
 
-int backend_commit(struct mailbox_transaction_context *ctx,
-		   struct antispam_transaction_context *ast)
+static int backend_commit(const struct antispam_config *cfg,
+			  struct mailbox_transaction_context *ctx,
+			  struct antispam_transaction_context *ast)
 {
 	struct siglist *item = ast->siglist;
 	int ret = 0;
 
 	while (item) {
-		if (call_dspam(item->sig, item->wanted)) {
+		if (call_dspam(cfg, item->sig, item->wanted)) {
 			ret = -1;
 			mail_storage_set_error(ctx->box->storage,
 					       ME(NOTPOSSIBLE)
@@ -200,9 +209,10 @@ int backend_commit(struct mailbox_transaction_context *ctx,
 	return ret;
 }
 
-int backend_handle_mail(struct mailbox_transaction_context *t,
-			struct antispam_transaction_context *ast,
-			struct mail *mail, enum classification want)
+static int backend_handle_mail(const struct antispam_config *cfg,
+			       struct mailbox_transaction_context *t,
+			       struct antispam_transaction_context *ast,
+			       struct mail *mail, enum classification want)
 {
 	const char *const *result = NULL;
 	int i;
@@ -211,57 +221,75 @@ int backend_handle_mail(struct mailbox_transaction_context *t,
 	 * Check for whitelisted classifications that should
 	 * be ignored when moving a mail. eg. virus.
 	 */
-	if (dspam_result_header)
-		result = get_mail_headers(mail, dspam_result_header);
+	if (cfg->dspam.result_header)
+		result = get_mail_headers(mail, cfg->dspam.result_header);
 	if (result && result[0]) {
-		for (i = 0; i < dspam_result_bl_num; i++) {
-			if (strcasecmp(result[0], dspam_result_bl[i]) == 0)
+		for (i = 0; i < cfg->dspam.result_bl_num; i++) {
+			if (strcasecmp(result[0], cfg->dspam.result_bl[i]) == 0)
 				return 0;
 		}
 	}
 
-	return signature_extract_to_list(t, mail, &ast->siglist, want);
+	return signature_extract_to_list(&cfg->dspam.sigcfg, t, mail, &ast->siglist, want);
 }
 
-void backend_init(pool_t pool)
+static void backend_init(struct antispam_config *cfg,
+			 const char *(getenv)(const char *env, void *data),
+			 void *getenv_data)
 {
 	const char *tmp;
 	int i;
 
-	tmp = get_setting("DSPAM_BINARY");
+	tmp = getenv("DSPAM_BINARY", getenv_data);
 	if (tmp)
-		dspam_binary = tmp;
-	debug("dspam binary set to %s\n", dspam_binary);
+		cfg->dspam.binary = tmp;
+	else
+		cfg->dspam.binary = "/usr/bin/dspam";
+	debug(&cfg->dbgcfg, "dspam binary set to %s\n", cfg->dspam.binary);
 
-	tmp = get_setting("DSPAM_RESULT_HEADER");
+	tmp = getenv("DSPAM_RESULT_HEADER", getenv_data);
 	if (tmp) {
-		dspam_result_header = tmp;
-		debug("dspam result set to %s\n", dspam_result_header);
+		cfg->dspam.result_header = tmp;
+		debug(&cfg->dbgcfg, "dspam result set to %s\n", cfg->dspam.result_header);
 
-		tmp = get_setting("DSPAM_RESULT_BLACKLIST");
+		tmp = getenv("DSPAM_RESULT_BLACKLIST", getenv_data);
 		if (tmp) {
-			dspam_result_bl = p_strsplit(pool, tmp, ";");
-			dspam_result_bl_num = str_array_length(
-					(const char *const *)dspam_result_bl);
-			for (i = 0; i < dspam_result_bl_num; i++)
-				debug("dspam result blacklist %s\n",
-						dspam_result_bl[i]);
+			cfg->dspam.result_bl = p_strsplit(cfg->mem_pool, tmp, ";");
+			cfg->dspam.result_bl_num = str_array_length(
+					(const char *const *)cfg->dspam.result_bl);
+			for (i = 0; i < cfg->dspam.result_bl_num; i++)
+				debug(&cfg->dbgcfg, "dspam result blacklist %s\n",
+				      cfg->dspam.result_bl[i]);
 		}
 	}
 
-	tmp = get_setting("DSPAM_ARGS");
+	tmp = getenv("DSPAM_ARGS", getenv_data);
 	if (tmp) {
-		extra_args = p_strsplit(pool, tmp, ";");
-		extra_args_num = str_array_length(
-					(const char *const *)extra_args);
-		for (i = 0; i < extra_args_num; i++)
-			debug("dspam extra arg %s\n",
-			      extra_args[i]);
+		cfg->dspam.extra_args = p_strsplit(cfg->mem_pool, tmp, ";");
+		cfg->dspam.extra_args_num = str_array_length(
+					(const char *const *)cfg->dspam.extra_args);
+		for (i = 0; i < cfg->dspam.extra_args_num; i++)
+			debug(&cfg->dbgcfg, "dspam extra arg %s\n",
+			      cfg->dspam.extra_args[i]);
 	}
 
-	signature_init();
+	tmp = getenv("DSPAM_ENV", getenv_data);
+	if (tmp) {
+		cfg->dspam.extra_env = p_strsplit(cfg->mem_pool, tmp, ";");
+		cfg->dspam.extra_env_num = str_array_length(
+					(const char *const *)cfg->dspam.extra_env);
+		for (i = 0; i < cfg->dspam.extra_env_num; i++)
+			debug(&cfg->dbgcfg, "dspam env %s\n",
+			      cfg->dspam.extra_env[i]);
+	}
+
+	signature_init(&cfg->dspam.sigcfg, &cfg->dbgcfg, getenv, getenv_data);
 }
 
-void backend_exit(void)
-{
-}
+struct backend dspam_backend = {
+	.init = backend_init,
+	.handle_mail = backend_handle_mail,
+	.start = backend_start,
+	.rollback = backend_rollback,
+	.commit = backend_commit,
+};
