@@ -104,47 +104,48 @@
 
 #include "antispam-plugin.h"
 
-static const char *spamspool = NULL;
-static const char * hamspool = NULL;
 
 struct antispam_transaction_context {
 	int count;
 };
 
 
-void backend_rollback(struct antispam_transaction_context *ast)
+static void backend_rollback(const struct antispam_config *cfg ATTR_UNUSED,
+			     struct antispam_transaction_context *ast)
 {
 	i_free(ast);
 }
 
-int backend_commit(struct mailbox_transaction_context *ctx __attr_unused__,
-		   struct antispam_transaction_context *ast)
+static int backend_commit(const struct antispam_config *cfg ATTR_UNUSED,
+			  struct mailbox_transaction_context *ctx ATTR_UNUSED,
+			  struct antispam_transaction_context *ast)
 {
 	i_free(ast);
 
 	return 0;
 }
 
-int backend_handle_mail(struct mailbox_transaction_context *t,
-			struct antispam_transaction_context *ast,
-			struct mail *mail, enum classification wanted)
+static int backend_handle_mail(const struct antispam_config *cfg,
+			       struct mailbox_transaction_context *t,
+			       struct antispam_transaction_context *ast,
+			       struct mail *mail, enum classification wanted)
 {
 	struct istream *mailstream;
 	struct ostream *outstream;
-	int ret;
+	int ret = -1;
 	const char *dest, *buf;
 	const unsigned char *beginning;
 	size_t size;
-	int fd;
+	int fd = -1;
 
 	i_assert(ast);
 
 	switch (wanted) {
 	case CLASS_SPAM:
-		dest = spamspool;
+		dest = cfg->s2d.spamspool;
 		break;
 	case CLASS_NOTSPAM:
-		dest =  hamspool;
+		dest = cfg->s2d.hamspool;
 		break;
 	default:	/* cannot handle this */
 		return -1;
@@ -157,8 +158,7 @@ int backend_handle_mail(struct mailbox_transaction_context *t,
 		return -1;
 	}
 
-	mailstream = get_mail_stream(mail);
-	if (!mailstream) {
+	if (mail_get_stream(mail, NULL, NULL, &mailstream) < 0) {
 		mail_storage_set_error(t->box->storage,
 				       ME(EXPUNGED)
 				       "Failed to get mail contents");
@@ -171,15 +171,16 @@ int backend_handle_mail(struct mailbox_transaction_context *t,
 	while (ast->count <= 9999) {
 		buf = t_strdup_printf(dest, (long)time(0), (long)++ast->count);
 		fd = open(buf, O_CREAT | O_EXCL | O_WRONLY, 0600);
-		if(fd >= 0 || errno != EEXIST)
+		if (fd >= 0 || errno != EEXIST)
 			break;
 		/* current filename in buf already exists, zap it */
 		t_pop();
 		t_push();
-		/* buf is invalid now ! */
+		/* buf is invalid now! */
 	}
+
 	if (fd < 0) {
-		debug("spool2dir backend: Failed to create spool file %s: %s\n",
+		debug(&cfg->dbgcfg, "spool2dir backend: Failed to create spool file %s: %s\n",
 		      dest, strerror(errno));
 		mail_storage_set_error(t->box->storage,
 				       ME(NOTPOSSIBLE)
@@ -188,6 +189,7 @@ int backend_handle_mail(struct mailbox_transaction_context *t,
 	}
 
 	/* buf still points to allocated memory, because fd >= 0 */
+
 	outstream = o_stream_create_from_fd(fd, t->box->pool);
 	if (!outstream) {
 		mail_storage_set_error(t->box->storage,
@@ -198,7 +200,6 @@ int backend_handle_mail(struct mailbox_transaction_context *t,
 
 	if (i_stream_read_data(mailstream, &beginning, &size, 5) < 0 ||
 	    size < 5) {
-		ret = -1;
 		mail_storage_set_error(t->box->storage,
 				       ME(NOTPOSSIBLE)
 				       "Failed to read mail beginning");
@@ -210,7 +211,6 @@ int backend_handle_mail(struct mailbox_transaction_context *t,
 		i_stream_read_next_line(mailstream);
 
 	if (o_stream_send_istream(outstream, mailstream) < 0) {
-		ret = -1;
 		mail_storage_set_error(t->box->storage,
 				       ME(NOTPOSSIBLE)
 				       "Failed to copy to spool file");
@@ -223,26 +223,30 @@ int backend_handle_mail(struct mailbox_transaction_context *t,
 	o_stream_destroy(&outstream);
  out_close:
 	close(fd);
- out:
 	if (ret)
 		unlink(buf);
-
+ out:
 	t_pop();
 
 	return ret;
 }
 
-void backend_init(pool_t pool __attr_unused__)
+static void backend_init(struct antispam_config *cfg,
+			 const char *(getenv)(const char *env, void *data),
+			 void *getenv_data)
 {
-	if((spamspool = get_setting("SPOOL2DIR_SPAM")))
-		debug("spool2dir spamspool %s\n", spamspool);
+	cfg->s2d.spamspool = getenv("SPOOL2DIR_SPAM", getenv_data);
+	if (cfg->s2d.spamspool)
+		debug(&cfg->dbgcfg, "spool2dir spamspool %s\n", cfg->s2d.spamspool);
 
-	if((hamspool = get_setting("SPOOL2DIR_NOTSPAM")))
-		debug("spool2dir hamspool %s\n", hamspool);
+	cfg->s2d.hamspool = getenv("SPOOL2DIR_NOTSPAM", getenv_data);
+	if (cfg->s2d.hamspool)
+		debug(&cfg->dbgcfg, "spool2dir hamspool %s\n", cfg->s2d.hamspool);
 }
 
-struct antispam_transaction_context *
- backend_start(struct mailbox *box __attr_unused__)
+static struct antispam_transaction_context *
+backend_start(const struct antispam_config *cfg ATTR_UNUSED,
+	      struct mailbox *box ATTR_UNUSED)
 {
 	struct antispam_transaction_context *ast;
 
@@ -254,6 +258,10 @@ struct antispam_transaction_context *
 }
 
 
-void backend_exit(void)
-{
-}
+struct backend spool2dir_backend = {
+	.init = backend_init,
+	.handle_mail = backend_handle_mail,
+	.start = backend_start,
+	.rollback = backend_rollback,
+	.commit = backend_commit,
+};
