@@ -1,6 +1,6 @@
 #!/bin/sh
 
-VERSION=1.2
+VERSION=2.0
 
 if head=$(git rev-parse --verify HEAD 2>/dev/null); then
 	git update-index --refresh --unmerged > /dev/null
@@ -10,12 +10,12 @@ if head=$(git rev-parse --verify HEAD 2>/dev/null); then
 	# is correct...
 	[ "${descr%%-*}" = "v$VERSION" ] || exit 2
 
-	echo -n '#define ANTISPAM_VERSION "'
-	echo -n "${descr#v}"
+	echo -n '#define ANTISPAM_VERSION "' > $1
+	echo -n "${descr#v}" >> $1
 	if git diff-index --name-only HEAD | read dummy ; then
-		echo -n "-dirty"
+		echo -n "-dirty" >> $1
 	fi
-	echo '"'
+	echo '"' >> $1
 else
-	echo '#define ANTISPAM_VERSION "'$VERSION'-notgit"'
+	echo '#define ANTISPAM_VERSION "'$VERSION'-notgit"' > $1
 fi
